#include "../include/game.h"
#include <stdio.h>

bool chunk_is_block_neighboring_block(Chunk* chunk, vec3i pos)
{
    if (is_oob(pos.x-1, 0, CHUNK_MAX_X) || is_oob(pos.x+1, 0, CHUNK_MAX_X))
        return false;
    if (is_oob(pos.y-1, 0, CHUNK_MAX_Y) || is_oob(pos.y+1, 0, CHUNK_MAX_Y))
        return false;
    if (is_oob(pos.z-1, 0, CHUNK_MAX_Z) || is_oob(pos.z+1, 0, CHUNK_MAX_Z))
        return false;

    if (chunk->blocks[pos.x-1][pos.y][pos.z].type != BLOCKTYPE_AIR &&
        chunk->blocks[pos.x+1][pos.y][pos.z].type != BLOCKTYPE_AIR &&
        chunk->blocks[pos.x][pos.y-1][pos.z].type != BLOCKTYPE_AIR &&
        chunk->blocks[pos.x][pos.y+1][pos.z].type != BLOCKTYPE_AIR &&
        chunk->blocks[pos.x][pos.y][pos.z-1].type != BLOCKTYPE_AIR &&
        chunk->blocks[pos.x][pos.y][pos.z+1].type != BLOCKTYPE_AIR)
        return true;

    return false;
}

void chunk_set_blocks_visibility(Chunk* chunk)
{
    for (int x = 0; x < CHUNK_MAX_X; x++) {
        for (int y = 0; y < CHUNK_MAX_Y; y++) {
            for (int z = 0; z < CHUNK_MAX_Z; z++) {
                vec3i v = {x, y, z};
                if (chunk_is_block_neighboring_block(chunk, v))
                    chunk->blocks[x][y][z].visible = false;
                else
                    chunk->blocks[x][y][z].visible = true;

                if (chunk->blocks[x][y][z].type == BLOCKTYPE_AIR)
                    chunk->blocks[x][y][z].visible = false;
            }
        }
    }
}

void chunk_create_displayl(App *app, Chunk *chunk)
{
    chunk->id = glGenLists(1);

    glNewList(chunk->id, GL_COMPILE);
        for (int x = 0; x < CHUNK_MAX_X; x++) {
            for (int y = 0; y < CHUNK_MAX_Y; y++) {
                for (int z = 0; z < CHUNK_MAX_Z; z++) {
                    if (chunk->blocks[x][y][z].visible)
                        draw_cube(app, chunk->start_pos.x + x, chunk->start_pos.y + y, chunk->start_pos.z + z);
                }
            }
        }
    glEndList();
}

void chunk_render(Chunk* chunk)
{
    glCallList(chunk->id);
}

void chunk_update(App *app, Chunk *chunk)
{
    chunk_set_blocks_visibility(chunk);
    chunk_create_displayl(app, chunk); 
}
